#include <iostream>
#include <unistd.h>
#include <cmath>
#include "simplex.hpp"

Complex operator+(Complex c1,Complex c2) {
  return Complex(c1._real+c2._real,c1._imag+c2._imag);
}
Complex operator*(Complex c1,Complex c2) {
  return Complex(c1._real*c2._real-c1._imag*c2._imag , c1._imag*c2._real+c2._imag*c1._real);
}
std::ostream& operator<<(std::ostream& os,Complex c) {
  os << c._real << " + " << c._imag << "i"; 
  return os;
}

template <>
Complex dotProd(std::vector<Complex>& u, std::vector<Complex>& v) {
  Complex res(0.,0.);
  for (int i=0;i<std::min(u.size(),v.size());i++)
    res = res + u[i]*v[i].conj();  
  return res;

}

// =============================================================
// Simplex class

template<int DIM>
Simplex<DIM>::Simplex(std::array<Point,DIM+1>& p) {

 if (DIM>3) {
   std::cerr << "Simplices are not implemented for dimensions above 3" << std::endl;
   abort();
 }

 _points = p;

}

template <>
double Simplex<0>::measure() {
  return 0;
}

template<>
double Simplex<1>::measure() {
  return sqrt(pow(_points[1][0]-_points[0][0],2)
             +pow(_points[1][1]-_points[0][1],2)
             +pow(_points[1][2]-_points[0][2],2));
}

template<>
double Simplex<2>::measure() {
  // Cross product of vectors P0P1 and P0P2
  double xp =  (_points[1][1]-_points[0][1])*(_points[2][2]-_points[0][2])
              -(_points[1][2]-_points[0][2])*(_points[2][1]-_points[0][1]);
  double yp = -(_points[1][0]-_points[0][0])*(_points[2][2]-_points[0][2])
              +(_points[1][2]-_points[0][2])*(_points[2][0]-_points[0][0]);
  double zp =  (_points[1][0]-_points[0][0])*(_points[2][1]-_points[0][1])
              -(_points[1][1]-_points[0][1])*(_points[2][0]-_points[0][0]);

  return 0.5*sqrt(xp*xp+yp*yp+zp*zp); 

}

template<>
double Simplex<3>::measure() {

  double det = 0.;

  std::array<std::array<double,3>,3> J;
  for (int i=0;i<3;i++)
  for (int j=0;j<3;j++)
    J[i][j] = _points[j+1][i]-_points[0][i];

  det = J[0][0]*(J[1][1]*J[2][2]-J[1][2]*J[2][1])
       -J[0][1]*(J[1][0]*J[2][2]-J[1][2]*J[2][0])
       +J[0][2]*(J[1][0]*J[2][1]-J[1][1]*J[2][0]);

  return 1./6.*fabs(det);

}

/* another solution (less "template-spirit")
template<int DIM>
double Simplex<DIM>::measure() {
  if (DIM==0) ...
  else if (DIM==1)...
  else if ...
}

*/

// Explicit instanciation of simplices
template class Simplex<0>;
template class Simplex<1>;
template class Simplex<2>;
template class Simplex<3>;

