#ifndef _SIMPLEX_HPP_
#define _SIMPLEX_HPP_

#include <array>
#include <vector>
#include <iostream>

using Point = std::array<double,3>;

class Complex {
  double _real,_imag;
  public:
    Complex(double r,double i) {_real=r;_imag=i;};
    Complex conj() {return Complex(_real,-_imag);};
    Complex& operator=(Complex c) {_real=c._real; _imag=c._imag; return *this;};
    friend Complex operator+(Complex c1,Complex c2);
    friend Complex operator*(Complex c1,Complex c2);
    friend std::ostream& operator<<(std::ostream& os,Complex c);
};
Complex operator+(Complex c1,Complex c2);
Complex operator*(Complex c1,Complex c2);
std::ostream& operator<<(std::ostream& os,Complex c);

// ================================================================
// non-class member template functions must be defined in the .hpp

template <typename T>
T dotProd(std::vector<T>& u, std::vector<T>& v) {

  T res(0.);
  for (int i=0;i<std::min(u.size(),v.size());i++)
    res = res + u[i]*v[i];  
  return res;

}

// But fully specialized templates must be in the .cpp to avoid
// multiple definitions
template <>
Complex dotProd(std::vector<Complex>& u, std::vector<Complex>& v);

// ================================================================

template <int DIM>
class Simplex {

  public :

    Simplex(std::array<Point,DIM+1>& p);

    double measure();

  private :
    std::array<Point,DIM+1> _points;

};

#endif // _SIMPLEX_HPP_

